const std = @import("std");

const Cubes = struct {
    red: u32 = 0,
    green: u32 = 0,
    blue: u32 = 0,
    fn isCompatibleWith(self: *Cubes, bag: Cubes) bool {
        //std.debug.print("bag: r{d}g{d}b{d}, current: r{d}g{d}b{d}\n", .{ bag.red, bag.green, bag.blue, self.red, self.green, self.blue });
        return self.red <= bag.red and self.green <= bag.green and self.blue <= bag.blue;
    }
};

fn solve(line: []const u8, bag: Cubes) u32 {
    //std.debug.print("\n", .{});
    var is_compatible = true;
    var game_number: u32 = 0;
    var current_number: u32 = 0;
    var current_pick = Cubes{};

    var index: usize = 0;
    while (index < line.len) : (index += 1) {
        const char = line[index];
        //std.debug.print("char: {s}\n", .{[_]u8{char}});
        switch (char) {
            '0'...'9' => {
                current_number = current_number * 10 + (char - '0');
            },
            'G' => {
                index += 4;
            },
            ':' => {
                game_number = current_number;
                current_number = 0;
            },
            ' ' => {},
            ',' => {},
            'r' => {
                current_pick.red = current_number;
                current_number = 0;
                index += 2;
            },
            'g' => {
                current_pick.green = current_number;
                current_number = 0;
                index += 4;
            },
            'b' => {
                current_pick.blue = current_number;
                current_number = 0;
                index += 3;
            },
            ';' => {
                is_compatible = is_compatible and current_pick.isCompatibleWith(bag);
                current_pick = Cubes{};
            },
            else => unreachable,
        }
    }

    is_compatible = is_compatible and current_pick.isCompatibleWith(bag);
    return if (is_compatible) game_number else 0;
}

pub fn main() !void {
    const stdout = std.io.getStdOut().writer();

    const raw_in = std.io.getStdIn();
    var buffered_reader = std.io.bufferedReader(raw_in.reader());
    var reader = buffered_reader.reader();

    const bag = Cubes{
        .red = 12,
        .green = 13,
        .blue = 14,
    };

    var result: u32 = 0;
    var line_buffer: [1000]u8 = undefined;
    while (try reader.readUntilDelimiterOrEof(&line_buffer, '\n')) |line| {
        result += solve(line, bag);
    }
    try stdout.print(
        "{d}\n",
        .{result},
    );
}

test "solve" {
    const bag = Cubes{
        .red = 12,
        .green = 23,
        .blue = 31,
    };
    try std.testing.expectEqual(@as(?u32, 76), solve("Game 76: 11 red, 22 green, 30 blue", bag));
    try std.testing.expectEqual(@as(?u32, 76), solve("Game 76: 12 red, 23 green, 31 blue", bag));
    try std.testing.expectEqual(@as(?u32, 0), solve("Game 76: 13 red, 23 green, 31 blue", bag));
    try std.testing.expectEqual(@as(?u32, 0), solve("Game 76: 12 red, 24 green, 31 blue", bag));
    try std.testing.expectEqual(@as(?u32, 0), solve("Game 76: 12 red, 23 green, 32 blue", bag));
    try std.testing.expectEqual(@as(?u32, 0), solve("Game 76: 13 red, 24 green, 32 blue", bag));
    try std.testing.expectEqual(@as(?u32, 76), solve("Game 76: 12 red, 23 green, 31 blue; 12 red, 23 green, 31 blue", bag));
    try std.testing.expectEqual(@as(?u32, 0), solve("Game 76: 13 red, 23 green, 31 blue; 12 red, 23 green, 31 blue", bag));
    try std.testing.expectEqual(@as(?u32, 0), solve("Game 76: 12 red, 23 green, 31 blue; 12 red, 23 green, 32 blue", bag));
}
