const std = @import("std");

fn StackList(comptime T: type, comptime capacity_type: type, comptime capacity: capacity_type) type {
    return struct {
        const Self = @This();
        mem: [capacity]T,
        length: capacity_type,

        fn add(self: *Self, value: T) void {
            self.mem[self.length] = value;
            self.length += 1;
        }

        fn has(self: *Self, needle: T) bool {
            for (0..self.length) |i| {
                if (self.mem[i] == needle) {
                    return true;
                }
            }

            return false;
        }

        fn init() Self {
            return Self{
                .mem = undefined,
                .length = 0,
            };
        }
    };
}

fn solve(line: []const u8) u32 {
    var result: u32 = 0;
    var winning_numbers = StackList(u16, u8, 20).init();

    var index: usize = 0;
    while (index < line.len and line[index] != ':') : (index += 1) {}

    index += 2;
    while (index < line.len and line[index] != '|') : (index += 3) {
        winning_numbers.add((@as(u16, line[index]) << 8) | @as(u16, line[index + 1]));
    }

    index += 2;
    while (index < line.len) : (index += 3) {
        const current_card = (@as(u16, line[index]) << 8) | @as(u16, line[index + 1]);
        if (winning_numbers.has(current_card)) {
            result = if (result == 0) 1 else result * 2;
        }
    }

    return result;
}

pub fn main() !void {
    const stdout = std.io.getStdOut().writer();

    const raw_in = std.io.getStdIn();
    var buffered_reader = std.io.bufferedReader(raw_in.reader());
    var reader = buffered_reader.reader();

    var result: u32 = 0;
    var line_buffer: [1000]u8 = undefined;
    while (try reader.readUntilDelimiterOrEof(&line_buffer, '\n')) |line| {
        result += solve(line);
    }
    try stdout.print("{d}\n", .{result});
}
